/* NIBObee BKit 2 Demo 
Es werden nur die IR Sensoren vom BKit2 benützt, nicht die Fühler.
Basierend auf einer Demo aus ROBOTER.CC
Bei starker Sonneneinstrahlung werden die IR-Transistoren gestört !!
Die Manöver sind auch vom Zustand der Batterien abhängig
mit den Werten speed_l... usw., delay und dist kann gespielt werden.
it's not perfect ;-)
*/

#include <nibobee/iodefs.h>
#include <nibobee/analog.h>
#include <nibobee/led.h>
#include <nibobee/delay.h>
#include <nibobee/clock.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <nibobee/motpwm.h>
#include <nibobee/sens.h>

#define AREF_VCC (_BV(REFS0) | _BV(REFS1))


// analogen Eingang messen
// mux ist Kanalnummer und die Refernzspannung

uint16_t analog_measure(uint8_t mux) {
  ADMUX = mux;
  ADCSRA = _BV(ADPS2)  // prescale faktor = 128 ADC laeuft 
         | _BV(ADPS1)  // mit 15 MHz / 128 = 120 kHz         
         | _BV(ADPS0)         
         | _BV(ADEN)   // ADC an         
         | _BV(ADSC);  // Beginne mit der Konvertierung  
         
  while (ADCSRA & _BV(ADSC)) {
    // Konvertierung laeuft
  }
  uint16_t result = ADC;
  return result;
}

#define LED_GREEN 0x02  // TXD X5.2 PD1
#define LED_RED 0x01    // RXD X5.1 PD0

// LEDs leuchten lassen:
// 0: aus
// 1: gruen
// 2: rot

void showState(uint8_t state) {
  PORTD &= (uint8_t)~0x03;
  switch (state) {
    case 0: PORTD |= 0x00; break;
    case 1: PORTD |= LED_GREEN; break;  // TXD X5.2 PD1
    case 2: PORTD |= LED_RED; break;    // RXD X5.1 PD0
  }
}

void setup() {

    led_init();                 // standard leds
    led_set(LED_L_YE, 0);
    led_set(LED_R_YE, 0);
}

#define T1 0x01  // PC0
#define T2 0x04  // PC2
#define T3 0x08  // PC3
#define T4 0x02  // PC1

#define IR_OFF 0x00
#define IR_A   0x08  // IR PORT PA3 
#define IR_B   0x04  // IR PORT PA2
#define IR_LED_AB ((1 << PA2) | (1 << PA3))  // beide Ports


// Transistor auswählen / Spalte selektieren (T1-T4)
void setupTransistor(uint8_t val) {
  PORTC |= (T1|T2|T3|T4);   // alle aus 
  PORTC &= ~val;            // löscht Bit
}

// IR-LED auswählen / Zeile selektieren (untere: IR_B, obere: IR_A, keine: IR_OFF)
void setupIR(uint8_t val) {
  PORTA |= (IR_A | IR_B);
  PORTA &= ~val;
}

// IR-Reflexion messen 
// i im Uhrzeigersinn: 0 (links hinten) bis 7 (rechts hinten)
uint16_t measure(uint8_t i) {
  
  setupIR(IR_OFF); // IR-LEDs ausschalten
  PORTA &= ~0x03; // Phototransitoren aktivieren PA0 + PA1
                    // löscht Bit 1 und 2 an Port A bzw. setzt auf 0
  switch (i) {
    case 0: setupTransistor(T1); break;
    case 1: setupTransistor(T1); break;
    case 2: setupTransistor(T2); break;
    case 3: setupTransistor(T2); break;
    case 4: setupTransistor(T4); break;
    case 5: setupTransistor(T4); break;
    case 6: setupTransistor(T3); break;
    case 7: setupTransistor(T3); break;
  }
  
  int16_t val = 0;
  switch (i) {
    case 1: case 3: case 4: case 6: // auf X1.1
      setupIR(IR_LED_AB);  // beide
      _delay_ms(1);
      val = analog_measure(0 | AREF_VCC);
      setupIR(IR_OFF);
      _delay_ms(1);
      val -= analog_measure(0 | AREF_VCC);
      break;
        
    case 0: case 2: case 5: case 7:  // auf X1.2
      setupIR(IR_LED_AB);  // beide
      _delay_ms(1);
      val = analog_measure(1 | AREF_VCC);
      setupIR(IR_OFF);
      _delay_ms(1);
      val -= analog_measure(1 | AREF_VCC);
      break;  
  }
  if (val<0) {
    val=0;
  }
  return val;
}

// ****************** Hauptschleife Main *******************

int main() {

  // Port-Pins als Ausgaenge konfigurieren
    DDRD |= (LED_GREEN|LED_RED);
    DDRA |= (IR_A|IR_B);
    DDRC |= (T1|T2|T3|T4);
    
    motpwm_init();
    uint8_t mode;
    int16_t r,l;
    motpwm_init();  // motorsetup
  	delay(200);
    int16_t speed_l=700;  // geradeaus
    int16_t speed_r=680;
    int16_t speed_l_rbogen=600;  // leichter bogen rechts
    int16_t speed_r_rbogen=300;
    int16_t speed_l_rechts=500;  // rechtskurve
    int16_t speed_r_rechts=350;
    int16_t speed_l_rdreh=600;  // rechts drehen
    int16_t speed_r_rdreh=450;    
    int16_t speed_l_rrueck=-350;  // rückwärts drehen rechts
    int16_t speed_r_rrueck=-600;
    int16_t speed_l_lrueck=-600;  // rückwärts drehen links
    int16_t speed_r_lrueck=-350;    
    int16_t speed_l_ldreh=450;  // links drehen
    int16_t speed_r_ldreh=600;
    int16_t speed_l_links=350;  // linkskurve
    int16_t speed_r_links=500;
    int16_t speed_l_lbogen=300;  // leichter bogen links
    int16_t speed_r_lbogen=600;
    int16_t speed_l_stop=0;  // Stop
    int16_t speed_r_stop=0;
    
    uint16_t left_bar = 0;  // Flag für Wand links
    uint16_t right_bar = 0;  // Flag für Wand rechts
    uint8_t general_dir = 0;  // bevorzugte Richtung

    
  // Standard-LEDs auch als Ausgaenge konfigurieren
  activate_output_group(IO_LEDS);
  
  // **************** Hauptschleife: ***********************
  
  while(1) {
  
    enable_interrupts();
    delay(1);
    motpwm_setLeft(speed_l);  // Geschwindigkeit setzen
    motpwm_setRight(speed_r); // geradeaus
    
    uint8_t clear=1;  // Flag für frei / nicht frei
    uint8_t dist=2;   // Abstand, wann detektiert wird

    sei();
    
    led_set(LED_L_YE, 0);
    led_set(LED_R_YE, 0);
    led_set(LED_L_RD, 0);
    led_set(LED_R_RD, 0);
    
    uint8_t i=0;
    
/* nötig
    if (left_bar > right_bar ) 
        { general_dir = 1;  // nach rechts
        }
        else 
        { general_dir = 0;  // nach links
        }
*/        
    if ( left_bar || right_bar > 100)

        {   left_bar = 0;
            right_bar = 0;
        }
 
    
//	uint8_t perform_check_IR(uint8_t mode) {

        // leichten Bogen nach rechts fahren
    for (uint8_t i=0; i<1; ++i) {  // i = 0  ist linkes hinten
      
         uint16_t value = measure(i);  // linke IRs abfragen, 1 = belegt
      
        if (value>dist) 					{
 
        			clear = 0;
        			led_set(LED_L_YE, 1);
                    motpwm_setLeft(speed_l_rbogen);
    				motpwm_setRight(speed_r_rbogen);
          			delay(300);
        							}
      							 }
    
    // leicht nach rechts fahren
    for (uint8_t i=1; i<2; ++i) {  // i = 1 ist links mitte
      
         uint16_t value = measure(i);  // linke IRs abfragen, 1 = belegt
      
        if (value>dist) 		{
 
        			clear = 0;
        			++left_bar;
        			led_set(LED_L_YE, 1);
        			led_set(LED_L_RD, 1);
                    motpwm_setLeft(speed_l_rechts);
    				motpwm_setRight(speed_r_rechts);
          			delay(300);
        							}
      							 }
      							 
      	//  stärker nach rechts						 
     for (uint8_t i=2; i<3; ++i) {  // i = 2 ist linkes vorne
      
         uint16_t value = measure(i);  // linke IRs abfragen, 1 = belegt
      
        if (value>dist)	{
 
        			clear = 0;
        			++left_bar;
        			led_set(LED_L_RD, 1);
                    motpwm_setLeft(speed_l_rdreh);
    				motpwm_setRight(speed_r_rdreh);
          			delay(400);
        							}
      							 }
//  MITTE  **********  rückwärts
        
     for (uint8_t i=3; i<5; ++i) {  // i = 3 und 4 ist mitte
        
             uint16_t value = measure(i);
                
        if (value>(dist+1)) 		{
          
              	clear = 0;
                showState(2);  // Rote LED an
 
            if (right_bar > left_bar )  // mehr rechte Wand als linke
                    {   motpwm_setLeft(speed_l_lrueck);		// noch etwas Drehung nach rechts
    			        motpwm_setRight(speed_r_lrueck);
                    }
                else 
                    {   motpwm_setLeft(speed_l_rrueck);		// noch etwas Drehung nach links
    			        motpwm_setRight(speed_r_rrueck);
    			     }   
				delay(800);
        								} 
          							}
          							
         //  stärker nach links 							
     for (uint8_t i=5; i<6; ++i) {  // i = 5 ist rechts vorne
      
         uint16_t value = measure(i);  // linke IRs abfragen, 1 = belegt
      
        if (value>dist) 	{   // detection sonst zu früh
 
        			clear = 0;
          			++right_bar;
        			led_set(LED_R_RD, 1);
                    motpwm_setLeft(speed_l_ldreh);
    				motpwm_setRight(speed_r_ldreh);
          			delay(400);
        							}
      							 }
      							 
      // leicht nach links							    
     for (uint8_t i=6; i<7; ++i) {  // i = 6 ist rechts mitte
        
                uint16_t value = measure(i);
                
        if (value>dist) 		{
                    clear = 0;
          			++right_bar;
              		led_set(LED_R_YE, 1);
                  	led_set(LED_R_RD, 1);
                    motpwm_setLeft(speed_l_links);
    				motpwm_setRight(speed_r_links);
          			delay(300);
         							}
                    }      							 
      // leicht nach links							    
     for (uint8_t i=7; i<8; ++i) {  // i = 7 ist rechts hinten
        
                uint16_t value = measure(i);
                
        if (value>dist) 			{
                    clear = 0;
              		led_set(LED_R_YE, 1);
                    motpwm_setLeft(speed_l_lbogen);
    				motpwm_setRight(speed_r_lbogen);
          			delay(300);
         							}
                    }
        
        // ende der Richtungswechsel            
    
        if (!clear) 				{
            showState(0);  // anhalten
            led_set(LED_L_YE, 0);
            led_set(LED_R_YE, 0);
            

    		        //motpwm_setLeft(-500);  // stottert zu oft durch anhalten wenn 0
    		        //motpwm_setRight(-500);  // fährt zu oft rückwärts wenn -500
        	        //delay(600);600						}						
      		_delay_ms(38);
    	
    	if (clear) {
      // alle Richtungen frei
            showState(1);  // gruen
            int16_t speed_l = (speed_l + 200); //etwas schneller fahren
            int16_t speed_r = (speed_r + 200);
            
            } else {
                    showState(0);
                    }
    }
  } return 0;
}
